From 9cd798a96ea2fab2da0eab450bc73359e22e5c63 Mon Sep 17 00:00:00 2001
From: Valentin Raevsky <valentin@compulab.co.il>
Date: Wed, 14 Dec 2016 14:36:02 +0200
Subject: ARM: i.MX6: dts: add ov564x mipi camera support

Signed-off-by: Valentin Raevsky <valentin@compulab.co.il>
---
 arch/arm/boot/dts/imx6qdl-cm-fx6.dtsi |  4 +++-
 arch/arm/boot/dts/imx6qdl-sb-fx6.dtsi | 41 +++++++++++++++++++++++++++++++++++
 arch/arm/mach-imx/mach-cm_fx6.c       | 14 ++++++++++++
 3 files changed, 58 insertions(+), 1 deletion(-)

diff --git a/arch/arm/boot/dts/imx6qdl-cm-fx6.dtsi b/arch/arm/boot/dts/imx6qdl-cm-fx6.dtsi
index 426e3c2..11789f2 100644
--- a/arch/arm/boot/dts/imx6qdl-cm-fx6.dtsi
+++ b/arch/arm/boot/dts/imx6qdl-cm-fx6.dtsi
@@ -228,6 +228,9 @@
 				MX6QDL_PAD_ENET_TXD1__GPIO1_IO29 0x80000000
 				/* PHY_RST */
 				MX6QDL_PAD_SD4_DAT0__GPIO2_IO08	0x80000000
+				/* mipi camera rst pwn */
+				MX6QDL_PAD_DI0_PIN4__GPIO4_IO20 0x80000000
+				MX6QDL_PAD_ENET_CRS_DV__GPIO1_IO25 0x80000000
 			>;
 		};
 	};
@@ -260,7 +263,6 @@
 				MX6QDL_PAD_DI0_PIN15__IPU1_DI0_PIN15       0x38
 				MX6QDL_PAD_DI0_PIN2__IPU1_DI0_PIN02        0x38
 				MX6QDL_PAD_DI0_PIN3__IPU1_DI0_PIN03        0x38
-				MX6QDL_PAD_DI0_PIN4__IPU1_DI0_PIN04        0x80000028
 				MX6QDL_PAD_DISP0_DAT0__IPU1_DISP0_DATA00   0x38
 				MX6QDL_PAD_DISP0_DAT1__IPU1_DISP0_DATA01   0x38
 				MX6QDL_PAD_DISP0_DAT2__IPU1_DISP0_DATA02   0x38
diff --git a/arch/arm/boot/dts/imx6qdl-sb-fx6.dtsi b/arch/arm/boot/dts/imx6qdl-sb-fx6.dtsi
index 22585fc..e98e819 100644
--- a/arch/arm/boot/dts/imx6qdl-sb-fx6.dtsi
+++ b/arch/arm/boot/dts/imx6qdl-sb-fx6.dtsi
@@ -53,6 +53,24 @@
 				pagesize = <16>;
 			};
 
+			ov564x_mipi: ov564x_mipi@3c {
+				compatible = "ovti,ov564x_mipi";
+				reg = <0x3c>;
+				clocks = <&clks IMX6QDL_CLK_CKO>;
+				clock-names = "csi_mclk";
+
+				DOVDD-supply = <&vgen4_reg>;
+				AVDD-supply = <&vgen3_reg>;
+				DVDD-supply = <&vgen2_reg>;
+
+				pwn-gpios = <&gpio1 25 1>;
+				rst-gpios = <&gpio4 20 0>;
+
+				csi_id = <0>;
+				mclk = <24000000>;
+				mclk_source = <0>;
+			};
+
 			adv5150: adv5150@5c {
 				compatible = "ti,tvp5150";
 				reg = <0x5c>;
@@ -95,6 +113,22 @@
 		#reset-cells = <0>;
 	};
 
+	vgen2_reg: vgen2 {
+		regulator-min-microvolt = <800000>;
+		regulator-max-microvolt = <1550000>;
+	};
+
+	vgen3_reg: vgen3 {
+		regulator-min-microvolt = <1800000>;
+		regulator-max-microvolt = <3300000>;
+	};
+
+	vgen4_reg: vgen4 {
+		regulator-min-microvolt = <1800000>;
+		regulator-max-microvolt = <3300000>;
+		regulator-always-on;
+	};
+
 	pcie_power_on_gpio: regulator-pcie-power-on-gpio {
 		compatible = "regulator-fixed";
 		regulator-name = "regulator-pcie-power-on-gpio";
@@ -196,3 +230,10 @@
 	status = "okay";
 };
 
+&mipi_csi {
+	status = "okay";
+	ipu_id = <0>;
+	csi_id = <0>;
+	v_channel = <0>;
+	lanes = <2>;
+};
diff --git a/arch/arm/mach-imx/mach-cm_fx6.c b/arch/arm/mach-imx/mach-cm_fx6.c
index 45d24cc..6b540f6 100644
--- a/arch/arm/mach-imx/mach-cm_fx6.c
+++ b/arch/arm/mach-imx/mach-cm_fx6.c
@@ -77,9 +77,23 @@ static void __init cm_fx6_csi_mux_init(void)
 	 * Set GPR13 bit 0-2 to 0x4.
 	 */
 	struct regmap *gpr;
+	struct device_node *np;
+	const char *status;
+	int ret;
+
+	/* Read mipi-csi status and set IPU1 CSI0 accordingly */
+	np = of_find_compatible_node(NULL, NULL, "fsl,imx6q-mipi-csi2");
+	if (np) {
+		ret = of_property_read_string(np, "status", &status);
+		if (!ret && (!strcasecmp(status, "okay"))) {
+			pr_info("IPU1 CSI0 is in mipi mode\n");
+			return;
+		}
+	}
 
 	gpr = syscon_regmap_lookup_by_compatible("fsl,imx6q-iomuxc-gpr");
 	if (!IS_ERR(gpr)) {
+		pr_info("IPU1 CSI0 is in parallel mode\n");
 		if (of_machine_is_compatible("fsl,imx6q"))
 			regmap_update_bits(gpr, IOMUXC_GPR1, 1 << 19, 1 << 19);
 		else if (of_machine_is_compatible("fsl,imx6dl"))
-- 
1.9.1

